
/**
   capacitive sensing code with servo motor for Crimson Fox
*/

#include <Servo.h>
#include <CapacitiveSensor.h>
#include <HampelFilter.h>
#include <Filters.h>

#define dir 5
#define stp 6
#define capin 8
#define capout 9
#define ledWork 14
#define ledPanic 15


//Initializing touch sensor
CapacitiveSensor   cap_sense = CapacitiveSensor(capin, capout);       // 10 megohm resistor between pins capin & capout

//Initialiying Filters
HampelFilter dataBuffer = HampelFilter(0.00, 3, 3.50);
FilterOnePole lowpassFilter( LOWPASS, 5);

//Initialiying threasholding for touch
int upperThreashold = 8000;
int lowerThreashold = 8000;
int baseline = 100;
int summedThreashold = 16000;
int csSum = 0;
long dynThreasholdCount = 0;
long dynThreasholdTimeout = 6000;

//Initializing Motor
int pos = 0;    // variable to store the servo position
bool exhale = false; // false for inhale, true for exhale
int breathDepthTop;
int breathDepthBottom;
int breathDelay;

//Initializing session and fox status
enum WorkingStatus {
  pause,
  work,
  panic,
  standby
};
WorkingStatus foxStatus;
bool touched = false; // status if touch was detected

//Initialize Timing
long sessionStart; //time at session start
long sessionLength = 30000; //180000ms = 3min, time after which fox panics, one hour would be 3600000, 30 min would be 1800000
long REACTIVITYDELAY = 2000;
long lastTouchTime;
long breathStepStart;

void setup()
{
  cap_sense.set_CS_AutocaL_Millis(0xFFFFFFFF);     // turn off autocalibrate on channel 1 - just as an example
  Serial.begin(9600);

  pinMode(stp, OUTPUT);
  pinMode(dir, OUTPUT);

  pinMode(ledWork, OUTPUT);
  pinMode(ledPanic, OUTPUT);

  enterPauseStatus();
  breathStepStart = millis();
  lastTouchTime = millis();
}

void loop() {

  //read and filter touch sensor signal
  long total1 =  lowpassFilter.input(cap_sense.capacitiveSensor(30));//30
  dataBuffer.write(total1);

/*
  if(!(foxStatus == standby)){
    Serial.print(lastTouchTime);
    Serial.print(", ");
    Serial.print(millis());
    Serial.print(", ");
    Serial.println(lastTouchTime + REACTIVITYDELAY - millis());
  }*/
  
  if ((lastTouchTime + REACTIVITYDELAY - millis()) > 4294000000) {
    //Serial.println("checking status");
    // state machine
    if (foxStatus == pause) {
      //enter work status if touched
      if (detectTouch(total1)) {
        enterWorkStatus();
      }
    } else if (foxStatus == work) {
      //inter pause status if touched
      if (detectTouch(total1)) {
        enterPauseStatus();
      }
      //check if it is time to panic
      if ((millis() - sessionStart) > sessionLength) {
        enterPanicStatus();
      }
    } else if(foxStatus == panic) { //panic
      //enter work status if touched
      if (detectTouch(total1)) {
        enterWorkStatus();
      }
    }
  }

  if(foxStatus != standby){
      breathing();
  }

  if(Serial.available() > 0){
    String userInput = Serial.readString();
    Serial.print(userInput);
    userInput.trim();
    if(userInput.equals("q") || userInput == "quit"){
      exhaleAndDie();
      foxStatus = standby;
    }else{
      Serial.println(userInput + " could not be recogninzed use 'q' or 'quit' to shut down the fox.");
    }
  }

}

bool detectTouch(int cs) {
  bool touchTriggered = false;
  if (!dataBuffer.checkIfOutlier(cs)) {
    Serial.print(cs);
    Serial.print("\t");
    if (cs > upperThreashold) {
      csSum += cs;
      Serial.print(cs);
      Serial.print("\t");
      if (csSum >= summedThreashold) //c: This value is the threshold, a High value means it takes longer to trigger
      {
        Serial.print("Trigger: ");
        Serial.print(csSum);
        touchTriggered = true;
        if (csSum > 0) {
          csSum = 0;  //Reset
        }
        //cs_7_8.reset_CS_AutoCal(); //Stops readings
      }
    } else if (cs < lowerThreashold) {
      csSum = 0; //Timeout caused by bad readings
    }
    Serial.println("");
  }
  return touchTriggered;
}

void breathing() {
  //Serial.println(millis()-breathStepStart);
  
  /*if(!(foxStatus == standby)){
    Serial.print(breathStepStart);
    Serial.print(", ");
    Serial.print(millis());
    Serial.print(", ");
    Serial.println(millis()-breathStepStart);
  }*/
  
  //if(breathDelay < (millis()-breathStepStart)){
    breathStepStart = millis();
    
    if (pos > breathDepthTop) {
      exhale = true;
    } else if (pos <= breathDepthBottom) {
      exhale = false;
    }
  
    //set motor direction
    if (exhale) {
      pos = pos - breathDelay;
      digitalWrite(dir, LOW);
    } else {
      pos = pos + breathDelay;
      //Serial.println(pos);
      digitalWrite(dir, HIGH);
    }
  
    //do a breath step before returning to measuring touch again
    for(int i = 0; i < breathDelay; i++){
      digitalWrite(stp, HIGH);
      delayMicroseconds(1);
      digitalWrite(stp, LOW);
      delayMicroseconds(1);
    }

  //}
}

void enterWorkStatus() {
  Serial.println("Enter work");
  sessionStart = millis();
  touched = false;
  foxStatus = work;
  breathDelay = 6;
  breathDepthTop = 800;
  breathDepthBottom = 0;
  digitalWrite(ledWork, HIGH); //TODO turn on again
  digitalWrite(ledPanic, LOW);
  lastTouchTime = millis();
}

void enterPanicStatus() {
  Serial.println("Enter panic");
  touched = false;
  foxStatus = panic;
  breathDelay = 10;
  breathDepthTop = 1500;
  breathDepthBottom = 1000;
  digitalWrite(ledWork, LOW);
  digitalWrite(ledPanic, HIGH);
}

void enterPauseStatus() {
  Serial.println("Enter pause");
  touched = false;
  foxStatus = pause;
  breathDelay = 2;
  breathDepthTop = 800; 
  breathDepthBottom = 0;
  digitalWrite(ledWork, LOW);
  digitalWrite(ledPanic, LOW);
  lastTouchTime = millis();
}

void exhaleAndDie(){
  digitalWrite(dir, LOW);
  while(pos > 0){    
    digitalWrite(stp, HIGH);
    delayMicroseconds(1);
    digitalWrite(stp, LOW);
    delayMicroseconds(1);
    pos--;
  }
}

void handleThreashold(){
  
}
}
